package ch.zhaw.projekt2.turnierverwaltung;

import ch.zhaw.projekt2.turnierverwaltung.main.gameScheduleView.GameController;
import ch.zhaw.projekt2.turnierverwaltung.main.gameScheduleView.GameDecorator;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.Pane;
import javafx.scene.layout.VBox;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class FactoryDecorator implements IsObservable{
    private Factory factory;
    private FileIO fileIO;
    private Pane pane;
    private List<IsObserver> listener = new ArrayList<>();

    public FactoryDecorator(FileIO fileIO, Factory factory, Pane pane){
        setFactory(factory);
        setFileIO(fileIO);
        setPane(pane);
    }

    public void setFileIO(FileIO fileIO) {
        this.fileIO = fileIO;
    }

    public void setFactory(Factory factory) {
        this.factory = factory;
    }

    public void setPane(Pane pane) {
        this.pane = pane;
    }

    @Override
    public void addListener(IsObserver observer) {
        listener.add(observer);
    }

    @Override
    public void removeListener(IsObserver observer) {
        listener.remove(observer);
    }

    public void openTournament(FileIO.TournamentFile tournamentFile){
        try {
            factory.setTournament(fileIO.loadTournament(tournamentFile));
            openScheduleView();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } //TODO handle and logging
    }

    public void openTournamentList() {
        factory.loadTournamentList((BorderPane) pane, this);
        informListener();
    }

    public void openParticipantFormular() {
        factory.loadParticipantFormular((BorderPane) pane, this);
        informListener();
    }

    public void openPlacesFormular() {
        factory.loadPlacesFormular((BorderPane) pane, this);
        informListener();
    }

    public void openScheduleView() {
        factory.loadGameScheduler((BorderPane) pane, this);
        informListener();
    }

    public GameController openGameView(VBox vBox, GameDecorator gameDecorator) {
        return factory.loadGameView(vBox ,gameDecorator, this);
    }

    public void informListener() {
        for(IsObserver observer : listener) {
            observer.update();
        }
    }
}
